from Observable import Observable

import time
import gtk
import gobject



class TimeWatcher(Observable):

    OBS_NEWDAY = 0
    OBS_SAVE_CONFIG = 1
    OBS_ALARM = 2



    def __init__(self):

        # (day, month, year)
        self.__today = (1, 1, 1)
        
        # table of alarms for the current day
        self.__alarms = {}

        gobject.idle_add(self.__watchtime)
        gobject.timeout_add(3000, self.__save_config)



    #
    # Reports when a new day starts.
    #
    def __watchtime(self):

        now = time.localtime(time.time())
        year, month, day = now[:3]


        if ((day, month, year) != self.__today):
            self.__today = (day, month, year)
            self.update_observer(self.OBS_NEWDAY, day, month, year)
            
        else:
            # check for alarms
            key = (now[3], now[4])
            if (key in self.__alarms):
                self.update_observer(self.OBS_ALARM, self.__alarms[key])
                del self.__alarms[key]
                    

        gobject.timeout_add(1000, self.__watchtime)



    #
    # Reports when the configuration should be saved.
    #
    def __save_config(self):

        self.update_observer(self.OBS_SAVE_CONFIG)
        return True


    #
    # Clears the currently set alarms.
    #
    def clear_alarms(self):
    
        self.__alarms.clear()


    #
    # Sets an alarm for the given time (seconds since epoch).
    #
    def set_alarm(self, hours, mins, obj):
    
        key = (hours, mins)
        if (not key in self.__alarms): self.__alarms[key] = []
        self.__alarms[key].append(obj)
        
