from code.ListBox import ListBox
from ical.CalEvent import CalEvent
from ical.Date import Date
from ical.RRule import RRule
from code import formats
from code import values

import gtk
from gtk import glade
import gobject
import os


#
# Editor for calendar events.
#
class CalEditor(object):

    __CMD_NEW = 0
    __CMD_REMOVE = 1


    #
    # Constructor.
    #
    def __init__(self):

        # the day
        self.__day = (0, 0, 0)

        # the calendar to use
        self.__calendar = None

        # the events on that day
        self.__events = []

        # the currently selected event
        self.__current_event = None

        # handler for the close action
        self.__close_handler = None

        # flag for indicating event blocking
        self.__events_blocked = 0

        # flag for indicating whether the user has changed an entry
        self.__changed_flag = 0


        gui = glade.XML(os.path.join(os.environ[values.DESKCALHOME],
                                     values.GLADEFILE))
        self.__window = gui.get_widget("caleditor")
        
        
        from code import icon; self.__window.set_icon(icon.ICON)
        self.__window.set_title(_("Edit Appointments"))
        self.__window.connect("delete-event", self.__on_close)

        # close button
        gui.get_widget("btn_close").connect("clicked", self.__on_close)

        self.__edit_frame = gui.get_widget("editframe")

        # list of appointments
        self.__listbox = ListBox(["Time", "Summary"])
        #self.__listbox.set_headers_visible(True)
        self.__listbox.connect_select_row(self.__on_select)
        self.__listbox.show()
        gui.get_widget("gap_listbox").add(self.__listbox)

        # buttons
        btn = gui.get_widget("btn_new")
        #btn.set_label(_("New Appointment"))
        btn.connect("clicked", self.__on_button, self.__CMD_NEW)

        btn = gui.get_widget("btn_remove")
        #btn.set_label(_("Remove Appointment"))
        btn.connect("clicked", self.__on_button, self.__CMD_REMOVE)

        # labels
        self.__events_title = gui.get_widget("lbl_events")
        gui.get_widget("lbl_edit").set_markup("<b>" + _("Edit Appointment") +
                                              "</b>")
        gui.get_widget("lbl_summary").set_text(_("Summary:"))
        gui.get_widget("lbl_time").set_text(_("Time:"))
        gui.get_widget("lbl_recurrences").set_text(_("Recurrences:"))
        gui.get_widget("lbl_reminder").set_text(_("Reminder:"))

        # edit widgets
        self.__entry_summary = gui.get_widget("entry_summary")
        self.__entry_summary.connect("changed", self.__on_text_change)

        self.__entry_hours = gui.get_widget("spin_hour")
        self.__entry_hours.connect("value-changed", self.__on_time_change, 0)

        self.__entry_mins = gui.get_widget("spin_minute")
        self.__entry_mins.connect("value-changed", self.__on_time_change, 1)

        text = _("every $INTERVAL$ ${days|weeks|months|years}$ for "
                 "$TIMES$ times")
        hbox = self.__create_recurrences_entry(text)
        gui.get_widget("gap_recurrence").add(hbox)
                
        self.__chk_reminder = gui.get_widget("chk_reminder")
        self.__chk_reminder.connect("toggled", self.__on_toggle_reminder)

        text = _("remind me $VALUE$ minutes earlier")
        hbox = self.__create_reminder_entry(text)
        gui.get_widget("gap_ante").add(hbox)


    def show(self):
    
        self.__window.show_all()



    #
    # Creates the recurrences entry.
    #
    def __create_recurrences_entry(self, data):

        hbox = gtk.HBox()
        hbox.show()

        parts = data.split("$")
        for p in parts:
            if (p == "INTERVAL"):
                spin = gtk.SpinButton(climb_rate = 1, digits = 0)
                spin.set_range(1, 1000)
                spin.set_increments(1, 10)
                spin.connect("value-changed", self.__on_rec_change, 0)
                spin.show()
                self.__entry_rec_interval = spin
                widget = spin
                
            elif (p == "TIMES"):
                spin = gtk.SpinButton(climb_rate = 1, digits = 0)
                spin.set_range(1, 1000)
                spin.set_increments(1, 10)
                spin.connect("value-changed", self.__on_rec_change, 2)
                spin.show()
                self.__entry_rec_times = spin
                widget = spin
                
            elif (p and p[0] == "{"):
                entries = p[1:-1].split("|")
                option = gtk.combo_box_new_text()
                option.show()
                menu = gtk.Menu()
                for e in entries:
                    option.append_text(e)
                #end for
                self.__entry_rec_freq = option
                option.connect("changed", self.__on_rec_change, 1)
                widget = option
                
            else:
                lbl = gtk.Label(p)
                lbl.show()
                widget = lbl

            hbox.pack_start(widget, False, False, 0)
        #end for
        
        return hbox
        
        
    #
    # Creates the reminder entry.
    #
    def __create_reminder_entry(self, data):

        hbox = gtk.HBox()
        hbox.show()

        parts = data.split("$")
        for p in parts:
            if (p == "VALUE"):
                spin = gtk.SpinButton(climb_rate = 1, digits = 0)
                spin.set_range(0, 999)
                spin.set_increments(1, 10)
                spin.connect("value-changed", self.__on_reminder_change)
                spin.show()
                self.__entry_reminder = spin
                widget = spin
            else:
                lbl = gtk.Label(p)
                lbl.show()
                widget = lbl

            hbox.pack_start(widget, False, False, 0)
        #end for
        
        return hbox

    #
    # Reacts on closing the window.
    #
    def __on_close(self, *args):
 
        self.__window.destroy()
        if (self.__changed_flag and self.__close_handler):
            self.__close_handler()

        return True



    #
    # Reacts on selecting an event.
    #
    def __on_select(self, src, row):

        self.__current_event = self.__events[row]
        summary = self.__current_event.get_summary()
        hours, mins, secs = self.__current_event.get_start().get_daytime()
        alarm = self.__current_event.get_alarm()

        self.__edit_frame.set_sensitive(True)
        self.__entry_summary.set_text(self.__current_event.get_summary())
        self.__entry_hours.set_value(hours)
        self.__entry_mins.set_value(mins)
        self.__chk_reminder.set_active(alarm >= 0)
        if (alarm >= 0): self.__entry_reminder.set_value(alarm)
        

        rrules = self.__current_event.get_recurrences().get_rules()
        if (rrules):
            rrule = rrules[0]
            freq = rrule.get_frequency()
            interval = rrule.get_interval()
            times = rrule.get_count()

            self.__entry_rec_interval.set_value(interval)
            self.__entry_rec_freq.set_active(freq)
            self.__entry_rec_times.set_value(times)
        else:
            self.__entry_rec_interval.set_value(1)
            self.__entry_rec_freq.set_active(0)
            self.__entry_rec_times.set_value(1)
        #end if



    #
    # Reacts on changing text.
    #
    def __on_text_change(self, src):

        if (self.__events_blocked): return

        text = src.get_text()
        if (self.__current_event):
            self.__current_event.set_summary(text)
            self.__changed_flag = 1

        self.__update_list()


    #
    # Reacts on changing time values.
    #
    def __on_time_change(self, src, field):

        if (self.__events_blocked): return
        
        value = src.get_value_as_int()
        cyear, cmonth, cday = self.__current_event.get_start().get_day()
        chour, cmin, csec = self.__current_event.get_start().get_daytime()
        if (field == 0):
            chour = value
        elif (field == 1):
            cmin = value

        dtstart = Date(cyear, cmonth, cday, chour, cmin, csec)
        self.__current_event.set_start(dtstart)

        self.__current_event.invalidate()
        self.__changed_flag = 1
        self.__update_list()



    #
    # Reacts on changing recurrence values.
    #
    def __on_rec_change(self, src, field):

        if (self.__events_blocked): return

        interval = self.__entry_rec_interval.get_value_as_int()
        freq = self.__entry_rec_freq.get_active()
        times = self.__entry_rec_times.get_value_as_int()

        rrules = self.__current_event.get_recurrences().get_rules()
        if (not rrules):
            rrule = RRule()
            self.__current_event.get_recurrences().add_rule(rrule)
        else:
            rrule = rrules[0]

        # reset all delta values
        rrule.set_delta(RRule.DAY, 0)
        rrule.set_delta(RRule.MONTH, 0)
        rrule.set_delta(RRule.YEAR, 0)
                
        if (freq == 0):
            rrule.set_frequency(RRule.DAY)
            rrule.set_delta(RRule.DAY, 1)
        elif (freq == 1):
            rrule.set_frequency(RRule.WEEK)
            rrule.set_delta(RRule.DAY, 7)
        elif (freq == 2):
            rrule.set_frequency(RRule.MONTH)
            rrule.set_delta(RRule.MONTH, 1)
        elif (freq == 3):
            rrule.set_frequency(RRule.YEAR)
            rrule.set_delta(RRule.YEAR, 1)

        rrule.set_interval(interval)
        rrule.set_count(times)
        rrule.set_until(None)

        self.__current_event.invalidate()
        self.__changed_flag = 1
        
        
    #
    # Reacts on toggling the reminder checkbox.
    #
    def __on_toggle_reminder(self, src):
    
        if (self.__events_blocked): return
        
        value = src.get_active()
        if (value):
            ante = self.__current_event.get_alarm()
            ante = max(0, ante)
            self.__current_event.set_alarm(ante)
            self.__entry_reminder.set_value(ante)
            self.__entry_reminder.set_sensitive(True)
            
        else:
            self.__current_event.set_alarm(-1)
            self.__entry_reminder.set_sensitive(False)
        self.__changed_flag = 1
        
        
    #
    # Reacts on changing the reminder time.
    #
    def __on_reminder_change(self, src):
    
        if (self.__events_blocked): return
        
        ante = self.__entry_reminder.get_value_as_int()
        self.__current_event.set_alarm(ante)
        self.__changed_flag = 1

    #
    # Reacts on pressing a command button.
    #
    def __on_button(self, src, cmd):

        if (self.__events_blocked): return

        year, month, day = self.__day

        if (cmd == self.__CMD_NEW):
            new_event = CalEvent()
            new_event.set_start(Date(year, month, day, 8, 0, 0))
            self.__calendar.add_event(new_event)
            self.__display_events()
            row = self.__events.index(new_event)
            self.__listbox.set_cursor(row)
            self.__changed_flag = 1

        elif (cmd == self.__CMD_REMOVE and self.__current_event):
            row = self.__listbox.get_cursor()
            self.__calendar.remove_event(self.__current_event)
            self.__current_event = None
            self.__edit_frame.set_sensitive(False)
            self.__display_events()
            row = min(row, len(self.__events) - 1)
            if (row >= 0): self.__listbox.set_cursor(row)
            self.__changed_flag = 1


    #
    # Displays the existing events.
    #
    def __display_events(self):

        year, month, day = self.__day
        events = self.__calendar.get_events(year, month, day)
        if (events): events.sort(events[0].daytime_comparator)

        self.__listbox.clear()
        self.__events = []
        for e in events:
            hour, min, sec = e.get_start().get_daytime()
            daytime = formats.DAYTIMEFORMAT % vars()
            self.__listbox.append(daytime, e.get_summary())
            self.__events.append(e)
        #end for



    #
    # Updates the events list.
    #
    def __update_list(self):

        summary = self.__current_event.get_summary()
        hour, min, sec = self.__current_event.get_start().get_daytime()
        daytime = formats.DAYTIMEFORMAT % vars() % vars()

        row = self.__listbox.get_cursor()
        self.__listbox.set(row, daytime, summary)
        self.__ensure_order()



    #
    # Makes sure that the events are always listed in chronological order.
    #
    def __ensure_order(self):

        if (self.__events):
            e = self.__events[0]
            l1 = tuple(self.__events)
            self.__events.sort(e.daytime_comparator)
            l2 = tuple(self.__events)

            if (l1 != l2):
                self.__display_events()
                row = self.__events.index(self.__current_event)
                self.__listbox.set_cursor(row)
                
        #end if



    #
    # Sets the day to edit.
    #
    def set_day(self, year, month, day, calendar):

        def unblock(self): self.__events_blocked = 0

        self.__events_blocked = 1
        
        self.__day = (year, month, day)
        self.__calendar = calendar
        self.__current_event = None
        self.__entry_summary.set_text("")
        self.__entry_hours.set_value(0)
        self.__entry_mins.set_value(0)

        date = formats.YEARDATEFORMAT % vars()
        self.__events_title.set_markup("<b>" +
                                  _("Your appointments on %(date)s") % vars() +
                                  "</b>")
        self.__edit_frame.set_sensitive(False)
        self.__display_events()

        gobject.idle_add(unblock, self)
        



    #
    # Connects an action handler to the close action.
    #
    def connect_close(self, handler):

        self.__close_handler = handler
